# -*- coding: utf-8 -*-
# @Author  : LG

model_dict = {
    'mobile_sam.pt':
        {
            'urls': [
                'https://huggingface.co/yateng/isat_checkpoints/resolve/main/mobile_sam.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/mobile_sam.pt'
            ],
            'memory': '534M',
            'bf16_memory': '390M',
            'params': '40M',
            'image_segment': True,
            'video_segment': False,
        },
    'sam_hq_vit_h.pth':
        {
            'urls': [
                'https://huggingface.co/lkeab/hq-sam/resolve/main/sam_hq_vit_h.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_hq_vit_h.pth'
            ],
            'memory': '6464M',
            'bf16_memory': '3378M',
            'params': '2.6G',
            'image_segment': True,
            'video_segment': False,
        },
    'sam_hq_vit_l.pth':
        {
            'urls': [
                'https://huggingface.co/lkeab/hq-sam/resolve/main/sam_hq_vit_l.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_hq_vit_l.pth'
            ],
            'memory': '5016M',
            'bf16_memory': '2634M',
            'params': '1.3G',
            'image_segment': True,
            'video_segment': False,
        },
    'sam_hq_vit_b.pth':
        {
            'urls': [
                'https://huggingface.co/lkeab/hq-sam/resolve/main/sam_hq_vit_b.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_hq_vit_b.pth'
            ],
            'memory': '3304M',
            'bf16_memory': '1762M',
            'params': '379M',
            'image_segment': True,
            'video_segment': False,
        },
    'sam_hq_vit_tiny.pth':
        {
            'urls': [
                'https://huggingface.co/lkeab/hq-sam/resolve/main/sam_hq_vit_tiny.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_hq_vit_tiny.pth'
            ],
            'memory': '598M',
            'bf16_memory': '392M',
            'params': '43M',
            'image_segment': True,
            'video_segment': False,
        },
    'sam_vit_h_4b8939.pth':
        {
            'urls': [
                'https://dl.fbaipublicfiles.com/segment_anything/sam_vit_h_4b8939.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_vit_h_4b8939.pth'
            ],
            'memory': '6462M',
            'bf16_memory': '3378M',
            'params': '2.6G',
            'image_segment': True,
            'video_segment': False,
        },
    'sam_vit_l_0b3195.pth':
        {
            'urls': [
                'https://dl.fbaipublicfiles.com/segment_anything/sam_vit_l_0b3195.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_vit_l_0b3195.pth'
            ],
            'memory': '5016M',
            'bf16_memory': '2634M',
            'params': '1.3G',
            'image_segment': True,
            'video_segment': False,
        },
    'sam_vit_b_01ec64.pth':
        {
            'urls': [
                'https://dl.fbaipublicfiles.com/segment_anything/sam_vit_b_01ec64.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_vit_b_01ec64.pth'
            ],
            'memory': '3302M',
            'bf16_memory': '1760M',
            'params': '375M',
            'image_segment': True,
            'video_segment': False,
        },
    'edge_sam.pth':
        {
            'urls': [
                'https://huggingface.co/spaces/chongzhou/EdgeSAM/resolve/main/weights/edge_sam.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/edge_sam.pth'
            ],
            'memory': '360M',
            'bf16_memory': '304M',
            'params': '38.8M',
            'image_segment': True,
            'video_segment': False,
        },
    'edge_sam_3x.pth':
        {
            'urls': [
                'https://huggingface.co/spaces/chongzhou/EdgeSAM/resolve/main/weights/edge_sam_3x.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/edge_sam_3x.pth'
            ],
            'memory': '360M',
            'bf16_memory': '304M',
            'params': '38.8M',
            'image_segment': True,
            'video_segment': False,
        },
    'sam-med2d_b.pth':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam-med2d_b.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam-med2d_b.pth'
            ],
            'memory': '1500M',
            'bf16_memory': '1050M',
            'params': '2.4G',
            'image_segment': True,
            'video_segment': False,
        },
    'sam2_hiera_large.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2_hiera_large.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2_hiera_large.pt'
            ],
            'memory': '3252M',
            'bf16_memory': '2008M',
            'params': '900M',
            'image_segment': True,
            'video_segment': True,
        },
    'sam2_hiera_base_plus.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2_hiera_base_plus.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2_hiera_base_plus.pt'
            ],
            'memory': '1992M',
            'bf16_memory': '1666M',
            'params': '324M',
            'image_segment': True,
            'video_segment': True,
        },
    'sam2_hiera_small.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2_hiera_small.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2_hiera_small.pt'
            ],
            'memory': '1522M',
            'bf16_memory': '1226M',
            'params': '185M',
            'image_segment': True,
            'video_segment': True,
        },
    'sam2_hiera_tiny.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2_hiera_tiny.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2_hiera_tiny.pt'
            ],
            'memory': '1504M',
            'bf16_memory': '1196M',
            'params': '156M',
            'image_segment': True,
            'video_segment': True,
        },
    'sam2.1_hiera_large.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2.1_hiera_large.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2.1_hiera_large.pt'
            ],
            'memory': '3328M',
            'bf16_memory': '2266M',
            'params': '900M',
            'image_segment': True,
            'video_segment': True,
        },
    'sam2.1_hiera_base_plus.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2.1_hiera_base_plus.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2.1_hiera_base_plus.pt'
            ],
            'memory': '2032M',
            'bf16_memory': '1682M',
            'params': '324M',
            'image_segment': True,
            'video_segment': True,
        },
    'sam2.1_hiera_small.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2.1_hiera_small.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2.1_hiera_small.pt'
            ],
            'memory': '1736M',
            'bf16_memory': '1268M',
            'params': '185M',
            'image_segment': True,
            'video_segment': True,
        },
    'sam2.1_hiera_tiny.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2.1_hiera_tiny.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2.1_hiera_tiny.pt'
            ],
            'memory': '1574M',
            'bf16_memory': '1194M',
            'params': '156M',
            'image_segment': True,
            'video_segment': True,
        },
}



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/media/lg/disk2/PycharmProjects/ISAT_with_segment_anything/ISAT/ui/plugin_manager_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(1280, 720)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget = QtWidgets.QWidget(Dialog)
        self.widget.setMaximumSize(QtCore.QSize(16777215, 60))
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.widget)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(20)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout.addWidget(self.widget)
        self.tableWidget = QtWidgets.QTableWidget(Dialog)
        self.tableWidget.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidget.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.tableWidget.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(5)
        self.tableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(4, item)
        self.tableWidget.horizontalHeader().setVisible(True)
        self.tableWidget.horizontalHeader().setCascadingSectionResizes(False)
        self.tableWidget.horizontalHeader().setDefaultSectionSize(150)
        self.tableWidget.verticalHeader().setVisible(False)
        self.tableWidget.verticalHeader().setCascadingSectionResizes(False)
        self.tableWidget.verticalHeader().setDefaultSectionSize(30)
        self.verticalLayout.addWidget(self.tableWidget)
        self.widget_4 = QtWidgets.QWidget(Dialog)
        self.widget_4.setMaximumSize(QtCore.QSize(16777215, 30))
        self.widget_4.setObjectName("widget_4")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.widget_4)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(1087, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.pushButton_close = QtWidgets.QPushButton(self.widget_4)
        self.pushButton_close.setObjectName("pushButton_close")
        self.horizontalLayout_3.addWidget(self.pushButton_close)
        self.verticalLayout.addWidget(self.widget_4)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Plugins"))
        self.label.setText(_translate("Dialog", "Plugins"))
        item = self.tableWidget.horizontalHeaderItem(0)
        item.setText(_translate("Dialog", "Activate"))
        item = self.tableWidget.horizontalHeaderItem(1)
        item.setText(_translate("Dialog", "Name"))
        item = self.tableWidget.horizontalHeaderItem(2)
        item.setText(_translate("Dialog", "Author"))
        item = self.tableWidget.horizontalHeaderItem(3)
        item.setText(_translate("Dialog", "Version"))
        item = self.tableWidget.horizontalHeaderItem(4)
        item.setText(_translate("Dialog", "Description"))
        self.pushButton_close.setText(_translate("Dialog", "&Close"))



# -*- coding: utf-8 -*-
# @Author  : LG
import os
from importlib.metadata import entry_points
from ISAT.ui.plugin_manager_dialog import Ui_Dialog
from PyQt5 import QtWidgets, QtCore, QtGui


class PluginManagerDialog(QtWidgets.QDialog, Ui_Dialog):
    def __init__(self, parent, mainwindow):
        super(PluginManagerDialog, self).__init__(parent)
        self.mainwindow = mainwindow
        self.setupUi(self)
        self.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
        self.tableWidget.resizeColumnsToContents()
        self.tableWidget.horizontalHeader().setSectionResizeMode(4, QtWidgets.QHeaderView.Stretch)
        self.tableWidget.setColumnWidth(0, 100)
        self.tableWidget.setColumnWidth(1, 250)
        self.tableWidget.setColumnWidth(2, 150)
        self.tableWidget.setColumnWidth(3, 150)

        self.plugin_dir = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'plugins')
        self.plugins = []

        self.pushButton_close.clicked.connect(self.close)
        self.load_plugins()

    def load_plugins(self):
        self.tableWidget.setRowCount(0)
        print('loading plugins')
        eps = entry_points().get("isat.plugins", [])
        for ep in eps:
            try:
                plugin_class = ep.load()
                plugin_instance = plugin_class()
                plugin_instance.init_plugin(self.mainwindow)
                self.plugins.append(plugin_instance)
                print('loaded plugin: ', plugin_instance.get_plugin_name())
            except Exception as e:
                print('failed to load plugin [{ep}]: ', e)

        self.update_gui()

    def update_gui(self):
        self.tableWidget.setRowCount(0)
        row = 0
        for plugin_instance in self.plugins:
            activate_checkbox = QtWidgets.QCheckBox()
            activate_checkbox.stateChanged.connect(plugin_instance.activate_state_changed)
            plugin_name_item = QtWidgets.QTableWidgetItem(plugin_instance.get_plugin_name())
            plugin_author_item = QtWidgets.QTableWidgetItem(plugin_instance.get_plugin_author())
            plugin_author_item.setTextAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
            plugin_version_item = QtWidgets.QTableWidgetItem(plugin_instance.get_plugin_version())
            plugin_version_item.setTextAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
            plugin_description_item = QtWidgets.QTableWidgetItem(plugin_instance.get_plugin_description())

            self.tableWidget.insertRow(self.tableWidget.rowCount())
            self.tableWidget.setCellWidget(row, 0, activate_checkbox)
            self.tableWidget.setItem(row, 1, plugin_name_item)
            self.tableWidget.setItem(row, 2, QtWidgets.QTableWidgetItem(plugin_author_item))
            self.tableWidget.setItem(row, 3, QtWidgets.QTableWidgetItem(plugin_version_item))
            self.tableWidget.setItem(row, 4, QtWidgets.QTableWidgetItem(plugin_description_item))

            row += 1

    def trigger_before_image_open(self, image_path):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.before_image_open_event(image_path)

    def trigger_after_image_open(self):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.after_image_open_event()

    def trigger_before_annotation_start(self):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.before_annotation_start_event()

    def trigger_after_annotation_created(self):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.after_annotation_created_event()

    def trigger_after_annotation_changed(self):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.after_annotation_changed_event()

    def trigger_before_annotations_save(self):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.before_annotations_save_event()

    def trigger_after_annotations_saved(self):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.after_annotations_saved_event()

    def trigger_after_sam_encode_finished(self, index):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.after_sam_encode_finished_event(index)

    def trigger_on_mouse_move(self, scene_pos):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.on_mouse_move_event(scene_pos)

    def trigger_on_mouse_release(self, scene_pos):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.on_mouse_release_event(scene_pos)

    def trigger_on_mouse_press(self, scene_pos):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.on_mouse_press_event(scene_pos)

    def trigger_on_mouse_pressed_and_mouse_move(self, scene_pos):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.on_mouse_pressed_and_mouse_move_event(scene_pos)

    def trigger_application_start(self):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.application_start_event()

    def trigger_application_shutdown(self):
        for plugin in self.plugins:
            if plugin.enabled:
                plugin.application_shutdown_event()


# -*- coding: utf-8 -*-
# @Author  : LG

from abc import ABC, abstractmethod
import numpy as np


class PluginBase(ABC):
    def __init__(self):
        self.enabled = False

    @abstractmethod
    def init_plugin(self, mainwindow):
        """
        接受主程序调用，进行初始化
        :param mainwindow: 传入mainwindow，便于访问ISAT主程序的各种方法与属性
        """
        pass

    @abstractmethod
    def enable_plugin(self):
        """
        启动插件
        :param mainwindow:
        :return:
        """
        pass

    @abstractmethod
    def disable_plugin(self):
        """
        禁用插件
        :param mainwindow:
        :return:
        """
        pass

    @abstractmethod
    def get_plugin_author(self) -> str:
        """
        获取插件作者
        :return:
        """
        pass

    @abstractmethod
    def get_plugin_version(self) -> str:
        """
        获取插件版本
        :return:
        """
        pass

    @abstractmethod
    def get_plugin_description(self) -> str:
        """
        获取插件描述
        :return:
        """
        pass

    def get_plugin_name(self) -> str:
        """
        获取插件名
        :return:
        """
        return self.__class__.__name__

    def activate_state_changed(self, checkbox_state):
        if checkbox_state:
            self.enable_plugin()
        else:
            self.disable_plugin()

    def before_image_open_event(self, image_path: str):
        """
        当图片打开前调用
        :param image_path: 图片路径
        :return:
        """
        pass

    def after_image_open_event(self):
        """
        当图片打开后调用
        :param:
        :return:
        """
        pass

    def before_annotation_start_event(self):
        """
        当开始标注前调用
        :return:
        """
        pass

    def after_annotation_created_event(self):
        """
        当标注创建完成后调用
        :return:
        """
        pass

    def after_annotation_changed_event(self):
        """
        当标注发生变化后调用，包括创建新标注，拖动顶点，移动多边形，改变多边形属性等
        :return:
        """
        pass

    def before_annotations_save_event(self):
        """
        当保存标注文件前调用
        :return:
        """
        pass

    def after_annotations_saved_event(self):
        """
        当保存标注文件后调用
        :return:
        """
        pass

    def after_sam_encode_finished_event(self, index):
        """
        当sam编码完成后调用
        注意ISAT在单独线程对前后图像进行预编码，通过self.mainwindow.seganythread.results_dict中获取编码结果
        :param index: 编码完成的图片index
        :return:
        """
        pass


    def on_mouse_move_event(self, scene_pos):
        """
        当鼠标移动时调用
        :param scene_pos: 坐标，限制在图像范围内
        :return:
        """
        pass

    def on_mouse_press_event(self, scene_pos):
        """
        当鼠标按下时调用
        :param scene_pos: 坐标，限制在图像范围内
        :return:
        """
        pass

    def on_mouse_release_event(self, scene_pos):
        """
        当鼠标释放时调用
        :param scene_pos: 坐标，限制在图像范围内
        :return:
        """
        pass

    def on_mouse_pressed_and_mouse_move_event(self, scene_pos):
        """
        当鼠标拖动时调用
        :param scene_pos: 坐标，限制在图像范围内
        :return:
        """
        pass

    def application_start_event(self):
        """
        软件启动后触发
        :return:
        """
        pass

    def application_shutdown_event(self):
        """
        软件关闭前触发
        :return:
        """
        pass
